
# q-expansion of newform 129.2.a.c, downloaded from the LMFDB on 16 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 129
weight = 2
poly_data = [-2, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1, 1], [-1, 0], [1, -1], [1, -2], [3, -1], [-5, 0], [-2, -2], [-1, 4], [6, 0], [3, 3], [4, 0], [-4, -2], [0, 4], [1, 0], [-1, 7], [0, 8], [2, -8], [-4, 2], [-6, -6], [6, -2], [2, 4], [4, 6], [-7, -1], [0, -6], [-1, 2], [0, -4], [-6, 6], [-9, 5], [-7, -6], [-7, -1], [-8, -2], [0, -4], [15, 1], [4, -8], [-4, -2], [16, 0], [0, 6], [5, 10], [-3, -1], [14, -4], [0, 6], [-15, 0], [2, -8], [14, 0], [-2, -14], [8, -12], [1, 6], [21, 0], [6, 12], [10, 12], [1, -7], [-3, -5], [-8, 4], [-13, 1], [15, 3], [12, 6], [16, 10], [-2, 16], [-6, 6], [0, 2], [-18, 0], [-6, -6], [-24, -6], [27, 1], [6, 12], [-4, -10], [17, -12], [-15, -6], [6, -10], [4, -12], [12, 6], [-6, 4], [18, 0], [-2, -12], [10, -10], [0, -10], [9, -3], [-13, -2], [-2, 2], [-22, -6], [-6, 16], [0, -12], [-9, -9], [-16, 4], [-10, 8], [-29, 7], [15, 7], [-8, -18], [0, 18], [5, -12], [-6, -2], [19, -13], [-4, 2], [0, 26], [-1, 12], [-12, -6], [22, -8], [-9, -1], [15, -12], [10, 0], [2, 14], [-26, 8], [6, -20], [-20, 2], [-13, -18], [28, 2], [0, -14], [40, 2], [37, -5], [20, 2], [-11, 12], [38, 0], [-36, 4], [14, -12], [-8, 4], [-12, -14], [-12, 6], [22, -2], [37, 1], [2, 16], [-29, -10], [28, 6], [20, 10], [-6, 12], [36, 0], [12, 12], [-1, -24], [6, -4], [13, 2], [-8, 24], [-5, -14], [10, -10], [9, -24], [-26, 2], [0, -10], [-19, 14], [3, -9], [-8, -24], [-12, -8], [-42, 10], [-12, 0], [22, -20], [32, 4], [25, 17], [16, -4], [-18, -18], [-15, 12], [-36, -2], [11, 6], [6, 26], [-22, 20], [4, 2], [16, -4], [-24, 6], [-2, 4], [2, 22], [-6, -6], [-29, -11], [-20, 0], [-12, 6], [27, 9], [-7, -17], [10, -18], [2, 28], [42, -12], [12, -16], [23, -12], [-54, 0]]
